@@ -26,10 +26,10 @@
 
 #include "collectd.h"
 
-#include "common.h"
 #include "configfile.h"
 #include "filter_chain.h"
 #include "plugin.h"
+#include "utils/common/common.h"
 #include "utils_complain.h"
 
 /*
@@ -218,6 +218,34 @@ static int fc_config_add_match(fc_match_t **matches_head, /* {{{ */
     ptr = ptr->next;
   }
 
+  if (ptr == NULL && IS_TRUE(global_option_get("AutoLoadPlugin"))) {
+    char plugin_name[DATA_MAX_NAME_LEN];
+
+    status = ssnprintf(plugin_name, sizeof(plugin_name), "match_%s",
+                       ci->values[0].value.string);
+    if ((status < 0) || ((size_t)status >= sizeof(plugin_name))) {
+      ERROR("Automatically loading plugin \"match_%s\" failed:"
+            " plugin name would have been truncated.",
+            ci->values[0].value.string);
+      return -1;
+    }
+
+    status = plugin_load(plugin_name, /* flags = */ 0);
+    if (status != 0) {
+      ERROR("Automatically loading plugin \"%s\" failed "
+            "with status %i.",
+            plugin_name, status);
+      return status;
+    }
+
+    ptr = match_list_head;
+    while (ptr != NULL) {
+      if (strcasecmp(ptr->name, ci->values[0].value.string) == 0)
+        break;
+      ptr = ptr->next;
+    }
+  }
+
   if (ptr == NULL) {
     WARNING("Filter subsystem: Cannot find a \"%s\" match. "
             "Did you load the appropriate plugin?",
@@ -277,6 +305,34 @@ static int fc_config_add_target(fc_target_t **targets_head, /* {{{ */
     ptr = ptr->next;
   }
 
+  if (ptr == NULL && IS_TRUE(global_option_get("AutoLoadPlugin"))) {
+    char plugin_name[DATA_MAX_NAME_LEN];
+
+    status = ssnprintf(plugin_name, sizeof(plugin_name), "target_%s",
+                       ci->values[0].value.string);
+    if ((status < 0) || ((size_t)status >= sizeof(plugin_name))) {
+      ERROR("Automatically loading plugin \"target_%s\" failed:"
+            " plugin name would have been truncated.",
+            ci->values[0].value.string);
+      return -1;
+    }
+
+    status = plugin_load(plugin_name, /* flags = */ 0);
+    if (status != 0) {
+      ERROR("Automatically loading plugin \"%s\" failed "
+            "with status %i.",
+            plugin_name, status);
+      return status;
+    }
+
+    ptr = target_list_head;
+    while (ptr != NULL) {
+      if (strcasecmp(ptr->name, ci->values[0].value.string) == 0)
+        break;
+      ptr = ptr->next;
+    }
+  }
+
   if (ptr == NULL) {
     WARNING("Filter subsystem: Cannot find a \"%s\" target. "
             "Did you load the appropriate plugin?",
@@ -693,7 +749,7 @@ static int fc_bit_write_invoke(const data_set_t *ds, /* {{{ */
 
 static int fc_init_once(void) /* {{{ */
 {
-  static int done = 0;
+  static int done;
   target_proc_t tproc = {0};
 
   if (done != 0)
